@@ -1023,6 +1023,15 @@ QString Formatting::complianceStringShort(const GpgME::Key &key)
     if (key.isExpired()) {
         return i18n("expired");
     }
+    if (key.isRevoked()) {
+        return i18n("revoked");
+    }
+    if (key.isDisabled()) {
+        return i18n("disabled");
+    }
+    if (key.isInvalid()) {
+        return i18n("invalid");
+    }
 
     return i18nc("As in not all user IDs are valid.", "not certified");
 }
@@ -1032,7 +1041,14 @@ QString Formatting::prettyID(const char *id)
     if (!id) {
         return QString();
     }
-    return QString::fromLatin1(id).toUpper().replace(QRegularExpression(QStringLiteral("(....)")), QStringLiteral("\\1 ")).trimmed();
+    QString ret = QString::fromLatin1(id).toUpper().replace(QRegularExpression(QStringLiteral("(....)")),
+                                                            QStringLiteral("\\1 ")).trimmed();
+    // For the standard 10 group fingerprint let us use a double space in the
+    // middle to increase readability
+    if (ret.size() == 49) {
+        ret.insert(24, QLatin1Char(' '));
+    }
+    return ret;
 }
 
 QString Formatting::origin(int o)
